# pylint: disable=invalid-name,missing-docstring
from collections import Counter
import os

import yaml


def duplicated_view_labels(explore):
    c = Counter(v.display_label() for v in explore.views)
    return {label: n for label, n in c.items() if n > 1}


def label_issues(label, acronyms=None, abbreviations=None):
    acronyms = [] if acronyms is None else acronyms
    abbreviations = [] if abbreviations is None else abbreviations
    def _contains_bad_acronym_usage(label, acronym):
        words = label.split(' ')
        # drop plural 's' from words
        if not acronym.lower().endswith('s'):
            words = [w if not w.endswith('s') else w[:-1] for w in words]
        return any(acronym.upper() == w.upper() and w == w.title() for w in words)

    def _contains_bad_abbreviation_usage(label, abbreviation):
        return any(abbreviation.lower() == k.lower() for k in label.split(' '))

    acronyms_used = [
        a.upper() for a in acronyms if _contains_bad_acronym_usage(label, a)
    ]
    abbreviations_used = [
        a.title() for a in abbreviations if _contains_bad_abbreviation_usage(label, a)
    ]
    return acronyms_used + abbreviations_used


def view_label_issues(explore, acronyms=None, abbreviations=None):
    acronyms = [] if acronyms is None else acronyms
    abbreviations = [] if abbreviations is None else abbreviations
    results = {}
    for v in explore.views:
        issues = label_issues(v.display_label(), acronyms, abbreviations)
        if issues == []:
            continue
        results[v.display_label()] = issues
    return results


def explore_label_issues(model, acronyms=None, abbreviations=None):
    acronyms = [] if acronyms is None else acronyms
    abbreviations = [] if abbreviations is None else abbreviations
    results = {}
    for e in model.explores:
        issues = label_issues(e.display_label(), acronyms, abbreviations)
        if issues == []:
            continue
        results[e.display_label()] = issues
    return results


def mismatched_view_name(view):
    if view.name == view.filename[: -len('.view.lkml')]:
        return False
    return True


def field_label_issues(view, acronyms=None, abbreviations=None):
    acronyms = [] if acronyms is None else acronyms
    abbreviations = [] if abbreviations is None else abbreviations
    results = {}
    for f in view.fields:
        if f.is_hidden:
            continue
        issues = label_issues(f.display_label(), acronyms, abbreviations)
        if issues == []:
            continue
        results[f.display_label()] = issues
    return results


def missing_timeframes(field, timeframes=None):
    timeframes = set() if timeframes is None else set(timeframes)
    field_timeframes = set()

    if field.is_hidden:
        return {}

    if hasattr(field, 'timeframes') and field.timeframes is not None:
        field_timeframes = set(field.timeframes)

    missing_timeframes = list(timeframes.difference(field_timeframes))

    return {'Missing Timeframe(s):': missing_timeframes} if missing_timeframes else {}


def field_missing_timeframes(view, timeframes=None):
    timeframes = [] if timeframes is None else timeframes
    results = {}
    for f in view.fields:
        if f.type not in ('date', 'date_time', 'time'):
            continue
        issues = missing_timeframes(f, timeframes)
        if not issues:
            continue
        results[f.display_label()] = issues
    return results


def derived_table_contains_semicolon(view):
    return view.derived_table_sql is not None and ';' in view.derived_table_sql


def unused_includes(model):
    # if all views in a project are imported into a model,
    # don't suggest any includes are unused
    if model.included_views == ['*']:
        return []
    explore_view_sources = [v.source_view.name for v in model.explore_views()]
    return sorted(list(set(model.included_views) - set(explore_view_sources)))


def read_lint_config(repo_path):
    # read .lintconfig.yml
    full_path = os.path.expanduser(repo_path)
    config_filepath = os.path.join(full_path, '.lintconfig.yml')
    acronyms = []
    abbreviations = []
    timeframes = []
    if os.path.isfile(config_filepath):
        with open(config_filepath) as f:
            config = yaml.load(f)
            acronyms = config.get('acronyms', acronyms)
            abbreviations = config.get('abbreviations', abbreviations)
            timeframes = config.get('timeframes', timeframes)
    lint_config = {'acronyms': acronyms, 'abbreviations': abbreviations, 'timeframes': timeframes}
    return lint_config


def lint_missing_timeframes(lkml, timeframes):
    if not timeframes:
        return {}

    # check for missing timeframe issues
    issues_field_dates = {}
    for v in lkml.views:
        issues = field_missing_timeframes(v, timeframes)
        if issues != {}:
            issues_field_dates[v.name] = issues
    return issues_field_dates


def lint_labels(lkml, acronyms, abbreviations):

    # check for acronym and abbreviation issues
    issues_explore_labels = {}
    for m in lkml.models:
        issues = explore_label_issues(m, acronyms, abbreviations)
        if issues != {}:
            issues_explore_labels[m.name] = issues
    issues_explore_view_labels = {}
    for m in lkml.models:
        for e in m.explores:
            issues = view_label_issues(e, acronyms, abbreviations)
            if issues != {}:
                if m.name not in issues_explore_view_labels:
                    issues_explore_view_labels[m.name] = {}
                issues_explore_view_labels[m.name][e.name] = issues
    issues_field_labels = {}
    for v in lkml.views:
        issues = field_label_issues(v, acronyms, abbreviations)
        if issues != {}:
            issues_field_labels[v.name] = issues

    # create overall labels issues dict
    all_label_issues = {}
    if issues_explore_labels != {}:
        all_label_issues['explores'] = issues_explore_labels
    if issues_explore_view_labels != {}:
        all_label_issues['explore_views'] = issues_explore_view_labels
    if issues_field_labels != {}:
        all_label_issues['fields'] = issues_field_labels
    return all_label_issues


def lint_duplicate_view_labels(lkml):
    issues = {}
    for m in lkml.models:
        for e in m.explores:
            dupes = duplicated_view_labels(e)
            if dupes == {}:
                continue
            if m.name not in issues:
                issues[m.name] = {}
            if e.name not in issues[m.name]:
                issues[m.name][e.name] = dupes
    return issues


def lint_sql_references(lkml):
    # check for raw SQL field references
    raw_sql_refs = {}
    for m in lkml.models:
        for e in m.explores:
            for v in e.views:
                if not v.contains_raw_sql_ref():
                    continue
                if m.name not in raw_sql_refs:
                    raw_sql_refs[m.name] = {}
                if e.name not in raw_sql_refs[m.name]:
                    raw_sql_refs[m.name][e.name] = {}
                raw_sql_refs[m.name][e.name][v.name] = v.sql_on
    return raw_sql_refs


def lint_view_primary_keys(lkml):
    # check for missing primary keys
    return [v.name for v in lkml.views if not v.primary_key()]


def lint_unused_includes(lkml):
    # check for unused includes
    return {
        m.name: unused_includes(m) for m in lkml.models if unused_includes(m) != []
    }


def lint_unused_view_files(lkml):
    # check for unused view files
    return lkml.unused_view_files()


def lint_missing_view_sql_definitions(lkml):
    return [
        v.name
        for v in lkml.views
        if not v.sql_definition()
        and v.extends == []
        and any(f.sql and '${TABLE}' in f.sql for f in v.fields)
    ]


def lint_semicolons_in_derived_table_sql(lkml):
    return [v.name for v in lkml.views if derived_table_contains_semicolon(v)]


def lint_mismatched_view_names(lkml):
    return {v.filename: v.name for v in lkml.views if mismatched_view_name(v)}



# pylint: disable=missing-docstring,invalid-name
import json

import click

from . import lookml
from . import lookmlint


CHECK_OPTIONS = [
    'all',
    'label-issues',
    'missing-timeframes',
    'raw-sql-in-joins',
    'unused-includes',
    'unused-view-files',
    'views-missing-primary-keys',
    'duplicate-view-labels',
    'missing-view-sql-definitions',
    'semicolons-in-derived-table-sql',
    'mismatched-view-names',
]


def _parse_checks(checks):
    checks = [c.strip() for c in checks.split(',')]
    for c in checks:
        if c not in CHECK_OPTIONS:
            raise click.BadOptionUsage(f'{c} not in {CHECK_OPTIONS}')
    if 'all' in checks:
        checks = list(set(CHECK_OPTIONS) - set(['all']))
    return sorted(checks)


def _run_check(check_name, lkml, lint_config):
    if check_name == 'label-issues':
        return lookmlint.lint_labels(
            lkml=lkml,
            acronyms=lint_config['acronyms'],
            abbreviations=lint_config['abbreviations'],
        )
    if check_name == 'missing-timeframes':
        return lookmlint.lint_missing_timeframes(
            lkml=lkml,
            timeframes=lint_config['timeframes'],
        )
    if check_name == 'raw-sql-in-joins':
        return lookmlint.lint_sql_references(lkml)
    if check_name == 'unused-includes':
        return lookmlint.lint_unused_includes(lkml)
    if check_name == 'unused-view-files':
        return lookmlint.lint_unused_view_files(lkml)
    if check_name == 'views-missing-primary-keys':
        return lookmlint.lint_view_primary_keys(lkml)
    if check_name == 'duplicate-view-labels':
        return lookmlint.lint_duplicate_view_labels(lkml)
    if check_name == 'missing-view-sql-definitions':
        return lookmlint.lint_missing_view_sql_definitions(lkml)
    if check_name == 'semicolons-in-derived-table-sql':
        return lookmlint.lint_semicolons_in_derived_table_sql(lkml)
    if check_name == 'mismatched-view-names':
        return lookmlint.lint_mismatched_view_names(lkml)
    raise Exception(f'Check: {check_name} not recognized')


def _format_output(check_name, results):
    lines = []
    if check_name == 'label-issues':
        if 'explores' in results:
            lines += ['Explores:']
            for model, model_results in results['explores'].items():
                lines.append(f'  Model: {model}')
                for explore, issues in model_results.items():
                    lines.append(f'    - {explore}: {issues}')
        if 'explore_views' in results:
            lines += ['Explore Views:']
            for model, model_results in results['explore_views'].items():
                lines.append(f'  Model: {model}')
                for explore, joins in model_results.items():
                    lines.append(f'    Explore: {explore}')
                    for join, issues in joins.items():
                        lines.append(f'      - {join}: {issues}')
        if 'fields' in results:
            lines += ['Fields:']
            for view, view_results in results['fields'].items():
                lines.append(f'  View: {view}')
                for field, issues in view_results.items():
                    lines.append(f'    - {field}: {issues}')
    if check_name == 'missing-timeframes':
        for view, view_results in results.items():
            lines.append(f'View: {view}')
            for field, issues in view_results.items():
                lines.append(f'  Field: {field}')
                for issue, timeframe in issues.items():
                    lines.append(f'   - {issue} {timeframe}')
    if check_name == 'raw-sql-in-joins':
        for model, model_results in results.items():
            lines.append(f'Model: {model}')
            for exploration, joins in model_results.items():
                lines.append(f'  Explore: {exploration}')
                for join, sql in joins.items():
                    lines.append(f'    {join}: {sql}')
    if check_name == 'unused-includes':
        for model, includes in results.items():
            lines.append(f'Model: {model}')
            for include in includes:
                lines.append(f'  - {include}')
    if check_name == 'unused-view-files':
        for view in results:
            lines.append(f'- {view}')
    if check_name == 'views-missing-primary-keys':
        for view in results:
            lines.append(f'- {view}')
    if check_name == 'duplicate-view-labels':
        for model, model_results in results.items():
            lines.append(f'Model: {model}')
            for exploration, joins in model_results.items():
                lines.append(f'  Explore: {exploration}')
                for join, num in joins.items():
                    lines.append(f'    {join}: {num}')
    if check_name == 'missing-view-sql-definitions':
        for view in results:
            lines.append(f'- {view}')
    if check_name == 'semicolons-in-derived-table-sql':
        for view in results:
            lines.append(f'- {view}')
    if check_name == 'mismatched-view-names':
        for view_file, view_name in results.items():
            lines.append(f'- {view_file}: {view_name}')
    return lines


@click.group('cli')
def cli():
    pass


@click.command('lint')
@click.argument('repo-path')
@click.option(
    '--checks',
    required=False,
    type=click.STRING,
    default='all',
    show_default=True,
    help='\n'.join(CHECK_OPTIONS),
)
@click.option('--json', 'json_output', is_flag=True, help='Format output as json')
def lint(repo_path, checks, json_output):
    checks = _parse_checks(checks)
    lkml = lookml.LookML(repo_path)
    lint_config = lookmlint.read_lint_config(repo_path)
    lint_results = {
        check_name: _run_check(check_name, lkml, lint_config) for check_name in checks
    }
    if json_output:
        click.echo(json.dumps(lint_results, indent=4))
    else:
        output_lines = []
        for check_name in sorted(lint_results.keys()):
            results = lint_results[check_name]
            if not (results == [] or results == {}):
                output_lines += ['\n', check_name, '-' * len(check_name)]
                output_lines += _format_output(check_name, results)

        if output_lines != []:
            raise click.ClickException('\n' + '\n'.join(output_lines) + '\n')


cli.add_command(lint)


if __name__ == '__main__':
    cli()



from . import lookml
import re


def date_issues(field, timeframes=None):
    timeframes = set() if timeframes is None else set(timeframes)
    field_timeframes = set()

    if field.is_hidden:
        return {}

    if hasattr(field, 'timeframes') and field.timeframes is not None:
        field_timeframes = set(field.timeframes)

    missing_timeframes = list(timeframes.difference(field_timeframes))
    fix_category = ''
    fix_category_id = 0

    if missing_timeframes:
        if hasattr(field, 'timeframes') and field.timeframes is not None:
            fix_category = 'Add additional timeframes'
            fix_category_id = 1
        if hasattr(field, 'timeframes') and field.timeframes is None:
            fix_category = 'Add all timeframes'
            fix_category_id = 2
        if not hasattr(field, 'timeframes'):
            fix_category = 'Change dimension to dimension_group and add all timeframes'
            fix_category_id = 3
        return {'fix_category': fix_category, 'fix_category_id': fix_category_id, 'current_timeframes': list(field_timeframes), 'missing_timeframes': missing_timeframes}

    return missing_timeframes

def add_additional_timeframes(view, field, timeframes_to_add):
    repo_path = "/Users/michael.shay/PycharmProjects/looker-bigquery/"
    view_to_edit = view + '.view.lkml'

    with open(repo_path + view_to_edit, 'r+') as file:
        view_text = file.read()

    start_of_field_attr = re.search("dimension_group(\s*):(\s*)" + field + "(\s*){",view_text).end()
    timeframes = re.compile("timeframes(\s*):(\s*)\[.*?\]", re.DOTALL)
    timeframes_to_update = timeframes.search(view_text[start_of_field_attr:len(view_text)]).span()
    view_text = view_text.replace(view_text[start_of_field_attr+timeframes_to_update[0]:start_of_field_attr+timeframes_to_update[1]], 'timeframes: [\n      ' + ',\n      '.join(timeframes_to_add) + '\n    ]')

    with open(repo_path + view_to_edit, 'w') as file:
        file.write(view_text)

def add_all_timeframes(view, field, timeframes_to_add):
    repo_path = "/Users/michael.shay/PycharmProjects/looker-bigquery/"
    view_to_edit = view + '.view.lkml'

    with open(repo_path + view_to_edit, 'r+') as file:
        view_text = file.read()

    start_of_field_attr = re.search("dimension_group(\s*):(\s*)" + field + "(\s*){",view_text).end()
    timeframes = re.compile("type(\s*):(\s*)time", re.DOTALL)
    timeframes_to_update = timeframes.search(view_text[start_of_field_attr:len(view_text)]).span()
    view_text = view_text.replace(view_text[start_of_field_attr+timeframes_to_update[0]:start_of_field_attr+timeframes_to_update[1]], 'type: time\n    timeframes: [\n      ' + ',\n      '.join(timeframes_to_add) + '\n    ]')

    with open(repo_path + view_to_edit, 'w') as file:
        file.write(view_text)


def fix_fields(view, timeframes=None):
    timeframes = [] if timeframes is None else timeframes
    results = {}
    for f in view.fields:
        if f.type not in ('date', 'date_time', 'time'):
            continue
        issues = date_issues(f, timeframes)
        if not issues:
            continue
        fix_category_id = issues.get('fix_category_id')
        current_timeframes = set(issues.get('current_timeframes'))
        missing_timeframes = set(issues.get('missing_timeframes'))
        timeframes_to_add = sorted(list(missing_timeframes.union(current_timeframes)))
        if fix_category_id == 1:
            add_additional_timeframes(view.name, f.name, timeframes_to_add)
        if fix_category_id == 2:
            add_all_timeframes(view.name, f.name, timeframes_to_add)
        if fix_category_id == 3:
            continue # manually fix these
        results[f.display_label()] = issues
    return results


def fix_dates():
    repo_path = "/Users/michael.shay/PycharmProjects/looker-bigquery"
    lkml = lookml.LookML(repo_path)
    timeframes = ['date', 'day_of_month', 'day_of_week', 'month', 'month_name', 'quarter', 'time', 'week', 'week_of_year', 'year']
    for v in lkml.views:
        fix_fields(v, timeframes)



# pylint: disable=invalid-name,missing-docstring,too-few-public-methods,too-many-instance-attributes
import os

import attr
import lkml


@attr.s
class Dimension:

    data = attr.ib(repr=False)
    name = attr.ib(init=False, repr=True)
    type = attr.ib(init=False)
    label = attr.ib(init=False)
    description = attr.ib(init=False, repr=False)
    sql = attr.ib(init=False, repr=False)
    is_primary_key = attr.ib(init=False)
    is_hidden = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.name = self.data['name']
        self.type = self.data.get('type', 'string')
        self.label = self.data.get('label')
        self.description = self.data.get('description')
        self.sql = self.data.get('sql')
        self.is_primary_key = self.data.get('primary_key') == 'yes'
        self.is_hidden = self.data.get('hidden') == 'yes'

    def display_label(self):
        return self.label if self.label else self.name.replace('_', ' ').title()


@attr.s
class DimensionGroup:

    data = attr.ib(repr=False)
    name = attr.ib(init=False, repr=True)
    type = attr.ib(init=False)
    label = attr.ib(init=False)
    description = attr.ib(init=False, repr=False)
    timeframes = attr.ib(init=False, repr=False)
    sql = attr.ib(init=False, repr=False)
    is_hidden = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.name = self.data['name']
        self.type = self.data.get('type', 'string')
        self.label = self.data.get('label')
        self.description = self.data.get('description')
        self.sql = self.data.get('sql')
        self.timeframes = self.data.get('timeframes')
        self.is_hidden = self.data.get('hidden') == 'yes'

    def display_label(self):
        return self.label if self.label else self.name.replace('_', ' ').title()


@attr.s
class Measure:

    data = attr.ib(repr=False)
    name = attr.ib(init=False, repr=True)
    type = attr.ib(init=False)
    label = attr.ib(init=False)
    description = attr.ib(init=False, repr=False)
    sql = attr.ib(init=False, repr=False)
    is_hidden = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.name = self.data['name']
        self.type = self.data.get('type')
        self.label = self.data.get('label')
        self.description = self.data.get('description')
        self.sql = self.data.get('sql')
        self.is_hidden = self.data.get('hidden') == 'yes'

    def display_label(self):
        return self.label if self.label else self.name.replace('_', ' ').title()


@attr.s
class View:

    data = attr.ib(repr=False)
    filename = attr.ib(repr=False)
    name = attr.ib(init=False)
    label = attr.ib(init=False)
    dimensions = attr.ib(init=False, repr=False)
    dimension_groups = attr.ib(init=False, repr=False)
    measures = attr.ib(init=False, repr=False)
    fields = attr.ib(init=False, repr=False)
    extends = attr.ib(init=False, repr=False)
    sql_table_name = attr.ib(init=False, repr=False)
    derived_table_sql = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.name = self.data['name']
        self.label = self.data.get('label')
        self.dimensions = [Dimension(d) for d in self.data.get('dimensions', [])]
        self.measures = [Measure(m) for m in self.data.get('measures', [])]
        self.dimension_groups = [
            DimensionGroup(dg) for dg in self.data.get('dimension_groups', [])
        ]
        self.fields = self.dimensions + self.dimension_groups + self.measures
        self.extends = [v.strip('*') for v in self.data.get('extends', [])]
        self.sql_table_name = self.data.get('sql_table_name')
        self.derived_table_sql = None
        if 'derived_table' in self.data:
            self.derived_table_sql = self.data['derived_table']['sql']

    def primary_key(self):
        try:
            return next(d.name for d in self.dimensions if d.is_primary_key)
        except StopIteration:
            return None
        finally:
            pass

    def sql_definition(self):
        priority = [self.sql_table_name, self.derived_table_sql]
        for v in priority:
            if v is not None:
                return v
        return None


@attr.s
class ExploreView:

    data = attr.ib(repr=False)
    explore = attr.ib()
    name = attr.ib(init=False)
    source_view = attr.ib(init=False, repr=False)
    from_view_name = attr.ib(init=False, repr=False)
    view_name = attr.ib(init=False)
    join_name = attr.ib(init=False)
    sql_on = attr.ib(init=False, repr=False)
    view_label = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.from_view_name = self.data.get('from')
        self.view_name = self.data.get('view_name')
        self.join_name = self.data.get('name')
        self.sql_on = self.data.get('sql_on')
        self.view_label = self.data.get('view_label')
        name_priority = [self.view_name, self.join_name, self.explore]
        self.name = next(v for v in name_priority if v is not None)

    def source_view_name(self):
        priority = [self.from_view_name, self.view_name, self.join_name, self.explore]
        return next(v for v in priority if v is not None)

    def display_label(self):
        priority = [
            self.view_label,
            self.source_view.label,
            self.name.replace('_', ' ').title(),
        ]
        return next(v for v in priority if v is not None)

    def contains_raw_sql_ref(self):
        if not self.sql_on:
            return False
        raw_sql_words = [
            w
            for line in self.sql_on.split('\n')
            for w in line.split()
            # not a comment line
            if not line.replace(' ', '').startswith('--')
            # doesn't contain lookml syntax
            and not '${' in w and not '}' in w
            # not a custom function with newlined args
            and not w.endswith('(')
            # contains one period
            and w.count('.') == 1
        ]
        return len(raw_sql_words) > 0


@attr.s
class Explore:

    data = attr.ib(repr=False)
    label = attr.ib(init=False)
    model = attr.ib()
    name = attr.ib(init=False)
    views = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.name = self.data.get('name')
        self.label = self.data.get('label')
        joined_views = [ExploreView(j, explore=self.name) for j in self.data.get('joins', [])]
        self.views = [ExploreView(self.data, explore=self.name)] + joined_views

    def display_label(self):
        return self.label if self.label else self.name.replace('_', ' ').title()


@attr.s
class Model:

    data = attr.ib(repr=False)
    filename = attr.ib(repr=False)
    explores = attr.ib(init=False, repr=False)
    included_views = attr.ib(init=False, repr=False)
    name = attr.ib(init=False)

    def __attrs_post_init__(self):
        includes = self.data.get('include', [])
        if isinstance(includes, str):
            includes = [includes]
        self.included_views = [i[: -len('.view')] for i in includes]
        self.name = self.filename[: -len('.model.lkml')]
        self.explores = [Explore(e, model=self.name) for e in self.data.get('explores', [])]

    def explore_views(self):
        return [v for e in self.explores for v in e.views]


@attr.s
class LookML:

    lookml_repo_path = attr.ib()
    models = attr.ib(init=False, repr=False)
    views = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):

        self.models = []
        for model_file in self._model_file_names():
            with open(os.path.join(self.lookml_repo_path, model_file)) as f:
                self.models.append(Model(lkml.load(f), model_file))
        self.views = []
        for view_file in self._view_file_names():
            with open(os.path.join(self.lookml_repo_path, view_file)) as f:
                for view_data in lkml.load(f)['views']:
                    self.views.append(View(view_data, view_file))

        # match explore views with their source views
        for m in self.models:
            for e in m.explores:
                for ev in e.views:
                    source_view = next(
                        v for v in self.views if v.name == ev.source_view_name()
                    )
                    ev.source_view = source_view

    def _view_file_names(self):
        return sorted(
            [f for f in os.listdir(self.lookml_repo_path) if f.endswith('.view.lkml')]
        )

    def _model_file_names(self):
        return sorted(
            [f for f in os.listdir(self.lookml_repo_path) if f.endswith('.model.lkml')]
        )

    def all_explore_views(self):
        explore_views = []
        for m in self.models:
            explore_views += m.explore_views()
        return explore_views

    def unused_view_files(self):
        view_names = [v.name for v in self.views]
        explore_view_names = [v.source_view.name for v in self.all_explore_views()]
        extended_views = [exv for v in self.views for exv in v.extends]
        return sorted(
            list(set(view_names) - set(explore_view_names) - set(extended_views))
        )



from setuptools import setup, find_packages

with open('readme.md') as f:
    long_description = f.read()

setup(
    name='lookmlint',
    description='Linter for LookML',
    version='0.2.3',
    author='Ryan Tuck',
    author_email='ryan.tuck@warbyparker.com',
    url='https://github.com/WarbyParker/lookmlint',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        'attrs',
        'click',
        'lkml==0.1.2',
        'pyyaml',
    ],
    packages=['lookmlint'],
    entry_points={
        'console_scripts': [
            'lookmlint = lookmlint.cli:cli',
        ]
    },
)
